package com.hdp.pi.utils.kolon;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.commons.net.ftp.FTPSClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;

import au.com.bytecode.opencsv.CSVReader;

public class FtpProductUtil {

	private static final Logger logger = LoggerFactory.getLogger(FtpProductUtil.class);

	@Value("${kolon.property.product.ftp.server}")
	private String ftpServer;

	@Value("${kolon.property.product.ftp.port}")
	private Integer ftpPort;

	@Value("${kolon.property.product.ftp.userName}")
	private String ftpUserName;

	@Value("${kolon.property.product.ftp.password}")
	private String ftpPassword;

	/**
	 * FTP获取文件目录
	 */
	@Value("${kolon.property.product.ftp.getFileDir}")
	private String getFileDir;

	@Value("${kolon.property.product.ftp.fileName}")
	private String fileNameFormat;
	
	/**
	 * FTP获取存放历史文件目录
	 */
	@Value("${kolon.property.product.ftp.getFileHistoryDir}")
	private String getFileHistoryDir;

	/**
	 * 获取ftp链接
	 * 
	 * @return ftpClient
	 * */
	public FTPClient getFTPClient() {
		FTPSClient ftpClient = null;
		try {
			ftpClient = new FTPSClient();
			ftpClient.connect(ftpServer, ftpPort);
			ftpClient.enterLocalPassiveMode();
			ftpClient.login(ftpUserName, ftpPassword);
			ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
			int reply = ftpClient.getReplyCode();
			if (!FTPReply.isPositiveCompletion(reply)) {
				logger.error("FTP server refused connection.");
				System.err.println("FTP server refused connection.");
				throw new RuntimeException("FTP server refused connection.");
			}
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
		return ftpClient;
	}
	
	/**
	 * 读取文件数据
	 */
	public List<String[]> readCSVFile(String fileName) {
		List<String[]> list = new ArrayList<String[]>();
		FTPClient ftpClient = null;
		try {
			ftpClient = this.getFTPClient();
			InputStream csv = ftpClient.retrieveFileStream(getFileDir
					+ fileName);
			if (csv == null) {
				logger.warn("kolon数据同步失败，没有找到【" + getFileDir + fileName
						+ "】数据文件");
				return null;
			}
			InputStreamReader inputStreamReader = new InputStreamReader(csv,
					"UTF-8");
			CSVReader csvReader = new CSVReader(inputStreamReader);
			list = csvReader.readAll();
			csvReader.close();
		} catch (Throwable e) {
			throw new RuntimeException(e);
		} finally{
			if(ftpClient!=null){
				try {
					ftpClient.disconnect();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		return list;
	}

	/**
	 * 移动文件
	 */
	public void moveFile(String fileName) {
		FTPClient ftpClient = null;
		try {
			ftpClient = this.getFTPClient();

			// 新的文件名
			String suffix = fileName.substring(fileName.lastIndexOf("."));
			String name = fileName
					.substring(0, fileName.lastIndexOf("."));
			String newFileName = name + "_" + getTime() + suffix;

			ftpClient.rename(getFileDir + fileName, getFileHistoryDir
					+ newFileName);
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}finally{
			if(ftpClient!=null){
				try {
					ftpClient.disconnect();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	private String getTime() {
		DateTimeFormatter formatter = DateTimeFormatter
				.ofPattern("yyyy-MM-dd_HHmmss");
		LocalDateTime dateTime = LocalDateTime.now();
		String formattedDateTime = dateTime.format(formatter);
		return formattedDateTime;
	}

	public void createdDirIfNotExist(String path) {
		File file = new File(path);
		if (!file.exists() && !file.isDirectory()) {
			file.mkdir();
		}
	}

	public String[] listNames(){
		String[] list = new String[0];
		FTPClient ftpClient = null;
		try {
			ftpClient = this.getFTPClient();
			list = ftpClient.listNames(getFileDir);
		} catch (Throwable e) {
			throw new RuntimeException(e);
		} finally{
			if(ftpClient!=null){
				try {
					ftpClient.disconnect();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		return list;
	}
	
	public boolean contains(String name){
		return name.contains(fileNameFormat);
	}
}
